library angular2_material.test.button_spec;

import "package:angular2/test_lib.dart"
    show
        AsyncTestCompleter,
        TestComponentBuilder,
        beforeEach,
        beforeEachBindings,
        ddescribe,
        describe,
        el,
        expect,
        iit,
        inject,
        it,
        xit;
import "package:angular2/src/core/debug/debug_element.dart" show DebugElement;
import "package:angular2/core.dart"
    show Component, View, ViewMetadata, UrlResolver, bind;
import "package:angular2_material/src/components/button/button.dart"
    show MdButton, MdAnchor;
import "test_url_resolver.dart" show TestUrlResolver;
import "package:angular2/src/core/compiler/xhr.dart" show XHR;
import "package:angular2/src/core/compiler/xhr_impl.dart" show XHRImpl;

main() {
  describe("MdButton", () {
    TestComponentBuilder builder;
    beforeEachBindings(() => [
          // Need a custom URL resolver for ng-material template files in order for them to work

          // with both JS and Dart output.
          bind(UrlResolver).toValue(new TestUrlResolver()),
          // Need to use the real XHR implementation (instead of the mock) so we can actually request

          // the template files, since Angular 2 doesn't have anything like $templateCache. This should

          // eventually be replaced with a preprocessor that inlines templates.
          bind(XHR).toClass(XHRImpl)
        ]);
    beforeEach(inject([TestComponentBuilder], (tcb) {
      builder = tcb;
    }));
    describe("button[md-button]", () {
      it(
          "should handle a click on the button",
          inject([AsyncTestCompleter], (async) {
            builder.createAsync(TestApp).then((rootTestComponent) {
              var testComponent =
                  rootTestComponent.debugElement.componentInstance;
              var buttonDebugElement = getChildDebugElement(
                  rootTestComponent.debugElement, "button");
              buttonDebugElement.nativeElement.click();
              expect(testComponent.clickCount).toBe(1);
              async.done();
            });
          }),
          1000);
      it(
          "should disable the button",
          inject([AsyncTestCompleter], (async) {
            builder.createAsync(TestApp).then((rootTestComponent) {
              var testAppComponent =
                  rootTestComponent.debugElement.componentInstance;
              var buttonDebugElement = getChildDebugElement(
                  rootTestComponent.debugElement, "button");
              var buttonElement = buttonDebugElement.nativeElement;
              // The button should initially be enabled.
              expect(buttonElement.disabled).toBe(false);
              // After the disabled binding has been changed.
              testAppComponent.isDisabled = true;
              rootTestComponent.detectChanges();
              // The button should should now be disabled.
              expect(buttonElement.disabled).toBe(true);
              // Clicking the button should not invoke the handler.
              buttonElement.click();
              expect(testAppComponent.clickCount).toBe(0);
              async.done();
            });
          }),
          1000);
    });
    describe("a[md-button]", () {
      const anchorTemplate =
          '''<a md-button href="http://google.com" [disabled]="isDisabled">Go</a>''';
      beforeEach(() {
        builder = builder.overrideView(TestApp,
            new ViewMetadata(template: anchorTemplate, directives: [MdAnchor]));
      });
      it(
          "should remove disabled anchors from tab order",
          inject([AsyncTestCompleter], (async) {
            builder.createAsync(TestApp).then((rootTestComponent) {
              var testAppComponent =
                  rootTestComponent.debugElement.componentInstance;
              var anchorDebugElement =
                  getChildDebugElement(rootTestComponent.debugElement, "a");
              var anchorElement = anchorDebugElement.nativeElement;
              // The anchor should initially be in the tab order.
              expect(anchorElement.tabIndex).toBe(0);
              // After the disabled binding has been changed.
              testAppComponent.isDisabled = true;
              rootTestComponent.detectChanges();
              // The anchor should now be out of the tab order.
              expect(anchorElement.tabIndex).toBe(-1);
              async.done();
            });
            it(
                "should preventDefault for disabled anchor clicks",
                inject([AsyncTestCompleter], (async) {
                  // No clear way to test this; see https://github.com/angular/angular/issues/3782
                  async.done();
                }));
          }),
          1000);
    });
  });
}

/** Gets a child DebugElement by tag name. */
DebugElement getChildDebugElement(DebugElement parent, String tagName) {
  return parent.query(
      (debugEl) => debugEl.nativeElement.tagName.toLowerCase() == tagName);
}

/** Test component that contains an MdButton. */
@Component(selector: "test-app")
@View(
    directives: const [MdButton],
    template:
        '''<button md-button type="button" (click)="increment()" [disabled]="isDisabled">Go</button>''')
class TestApp {
  num clickCount = 0;
  bool isDisabled = false;
  increment() {
    this.clickCount++;
  }
}
